/**
  *  \file server/getquery.cc
  */

#include <memory>
#include "server/getquery.h"
#include "server/queryout.h"
#include "game/struct.h"
#include "util/misc.h"
#include "server/serverio.h"
#include "server/jsonout.h"

static ServerQueryWriter*
createQueryWriter(string_t name)
{
    int n;
    if (name.size() > 6 && name.compare(0, 6, "shipfc", 6) == 0 && stringValue(name.substr(6), n) && n > 0 && n <= NUM_SHIPS) {
        return new ServerShipFCWriter(n);
    } else if (name.size() > 8 && name.compare(0, 8, "planetfc", 8) == 0 && stringValue(name.substr(8), n) && n > 0 && n <= NUM_PLANETS) {
        return new ServerPlanetFCWriter(n);
    } else if (name.size() > 7 && name.compare(0, 7, "shipmsn", 7) == 0 && stringValue(name.substr(7), n) && n > 0 && n <= NUM_SHIPS) {
        return new ServerShipMissionWriter(n);
    } else {
        return 0;
    }
}

/** Perform "GET query/..." command. */
void
performGetQuery(string_t query)
{
    std::auto_ptr<ServerQueryWriter> writer(createQueryWriter(query));

    // Generate output
    if (!writer.get()) {
        writeClient("404 Not found");
    } else {
        ServerJsonWriter w;
        writeClient("200 OK");
        w.startObject();
        writer->write(w);
        w.endObject();
        w.flush();
        writeClientTerminator();
    }
}
